/* Dump ROMfs contents.
 *
 * Copyright (C)2004 by Valentin Hilbig
 * GNU GPL V2 or higher.
 *
 * Just pipe the romfs into this here.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef unsigned long	QUAD;

static unsigned char	*buf;
static QUAD		already, pos, filled, sized, lastpos;

static void
ex(const char *s)
{
  perror(s);
  exit(1); abort(); for(;;);
}

/* not enough in buffer
 * extend it
 */
static int
fill(int min)
{
  int	got;

  if (pos && pos<filled)
    {
      memmove(buf, buf+pos, filled-=pos);
      already	+= pos;
      pos	= 0;
    }
  if (filled+min>=sized)
    {
      sized	= ((filled+min)/BUFSIZ+1)*BUFSIZ;
      if (sized<filled+min)
	ex("oops");
      buf	= realloc(buf, sized);
      if (!buf)
	ex("out of memory");
    }
  got	= fread(buf+filled, 1, sized-filled, stdin);
  if (got<0)
    ex("read error");
  filled	+= got;
  return got;
}

#define CURRENT()	(already+pos)
#define EAT(X)		lastpos=pos, pos += (X)
#define	BYTE(X)		(QUICK(X), buf[pos+(X)])
#define QUICK(X)	( filled<=pos+(X) ? fillmore((X)) : (void)0 )
static void
fillmore(int min)
{
  do
    if (filled>pos+min)
      return;
  while (fill(pos+min-filled));
  ex("unexpected EOF");
}

static QUAD
quad(int offs)
{
  return  (((QUAD)BYTE(offs+3))<<0)
	| (((QUAD)BYTE(offs+2))<<8)
	| (((QUAD)BYTE(offs+1))<<16)
	| (((QUAD)BYTE(offs+0))<<24)
	;
}

static int
pad(i)
{
  int	n;

  n	=  (already+pos+i)&15;
  if (n)
    n	= 16-n;
  while (n)
    {
      if (BYTE(i))
	{
	  printf("warning: padding not NUL at %lu\n", CURRENT()+i);
	  break;
	}
      i++;
      n--;
    }
  return i+n;
}

static const char *
string(int off)
{
  int	i;

  for (i=off; BYTE(i); i++);
  i	=  pad(i+1);
  off	+=  pos;
  EAT(i);
  return buf+off;
}

static void
checksum(int off, int size)
{
  QUAD	sum;
  int	i;

  off	-= pos;
  sum	= 0;
  for (i=0; i<size; i+=4, off+=4)
    sum	+= quad(off);
  if (sum)
    ex("checksum error");
}

static const char *
getsymlink(QUAD len)
{
  return buf+pos;
}

static QUAD maxpos;

static void
setmax(QUAD pos)
{
  if (maxpos<pos)
    maxpos	= pos;
}

static char *
my_strdup(const char *s)
{
  char *d;

  d	= strdup(s);
  if (!d)
    ex("out of memory");
  return d;
}

static int
file(void)
{
  QUAD		addr, next, spec, size, tmp;
  int		exec, mode;
  char *	name;

#define	MAJR	(unsigned)((spec>>16)&0xffff)
#define	MINR	(unsigned)((spec>>0)&0xffff)

  addr	= CURRENT();
  next	= quad(0);

  exec	= next&8;
  mode	= next&7;
  next	&= ~(QUAD)15;

  spec	= quad(4);
  size	= quad(8);
  name	= my_strdup(string(16));
  checksum(lastpos, pos-lastpos);

  if (size && (mode!=2 && mode!=3))
    printf("warning: size not 0:\n");

  if (next && next<CURRENT()+size)
    printf("warning: sequence error\n");

  tmp	= pad(size);

  printf("%8lx %c ", addr, (exec ? 'x' : '-'));
  switch (mode)
    {
    case 0:
      printf("hardlink [%8lx] %s\n", spec, name);
      break;

    case 1:
      setmax(spec);
      printf("dir      [%8lx] %s\n", spec, name);
      break;

    case 2:
      printf("file %13ld  %s\n", size, name);
      break;
      
    case 3:
      printf("symlink             %s -> %s\n", name, getsymlink(size));
      break;

    case 4:
      printf("block %4d,%4d  %s\n", MAJR, MINR, name);
      break;

    case 5:
      printf("char  %4d,%4d  %s\n", MAJR, MINR, name);
      break;

    case 6:
      printf("socket              %s\n", name);
      break;

    case 7:
      printf("pipe                %s\n", name);
      break;
    }
  if (next && next!=CURRENT()+tmp)
    {
      printf("           next=%lx\n\n", next);
      setmax(next);
    }
  EAT(tmp);
  free(name);
  if (next)
    return 1;

  printf("\n");
  return CURRENT()<=maxpos ? 2 : 0;
}

int
main(int argc, char **argv)
{
  QUAD	max, addr;
  char	*vol;

  if (fill(16)<16)
    ex("file too small");

  if (memcmp(buf, "-rom1fs-", 8))
    ex("magic error");

  checksum(0, 512);

  max	= quad(8);
  addr	= CURRENT()+16;

  vol	= my_strdup(string(16));

  printf("%8lx   volid               %s\n\n", addr, vol);

  while (file());

  printf("%8lx   volid               %s\n\n", addr, vol);
  printf("%8lx   fs_end\n", CURRENT());
  printf("%8lx   fs_size\n", max);
  
  if (CURRENT()>max)
    printf("Warning: fs end bejond size\n");

  if (CURRENT()<max)
    pad(max-CURRENT());

  return 0;
}
