#!/usr/bin/perl
#
# HTTPS upload of floppy image into an ERIC RMC card ( http://www.peppercon.de/ )
#
# Copyright (C)2004 by Valentin Hilbig
# Part of the HILBIX distribution, http://www.hilbix.com/
# This is free sostware according to GNU GPL V2 or higher.
#
# Tested with PERL 5.6.1 and
# ERIC Firmware 03.00.04 Build 501 Virtual Floppy 008 
#
# Usage:
#	eric-floppy.pl TARGET.img ericIP:ericPORT user pass [boot]
# boot defaults to always (other values: once no)
#
# This uses a file based cookie-jar ~/.eric-cookies as
# the firmware only allows one single user to upload the floppy image.

$floppy  = "vfloppy.asp";
$auth    = "auth.asp";

$fn = $ARGV[0];
$ip = $ARGV[1];
$us = $ARGV[2];
$pw = $ARGV[3];
$bo = $ARGV[4];
$bo = "always" if $bo eq "";

if ($#ARGV<3) {
    print "Usage: $0 IMAGE IP:port USER PASS [mode]\n";
    print "\tmode can be: no once always (latter default)\n";
    exit(1);
};

#
# Switch off SSL verification, as (my) ERIC does not have an official certificate.
# See http://20k.de/tn.php?s50
#
use IO::Socket::SSL;
IO::Socket::SSL::context_init({SSL_use_cert=>0,SSL_verify_mode=>0x00});

use LWP::UserAgent;
#use HTTP::Request;
#use HTTP::Request::Common;
use HTTP::Cookies;

#print "LWP-Version: $LWP::VERSION\n";

$cookies = HTTP::Cookies->new(
			      file		=> $ENV{HOME} . "/.eric-cookies",
			      autosave		=> 1,
			      ignore_discard	=> 1,
			      );

$ua = LWP::UserAgent->new(cookie_jar=>$cookies);
$ua->agent("HilbiX-eric/0.1 " . $ua->agent);

die("https not supported") unless $ua->is_protocol_supported("https");

print "ERIC UPLOAD $fn TO $ip AS $us MODE $bo\n";

print "connect .. "; $|=1; $|=0;

$url = "https://$ip/$floppy";
$res = $ua->get($url);
die("error $url:\n" . $res->code . " " . $res->message) unless ($res->is_success);

if ($res->content =~ $auth) {
    print "login .. "; $|=1; $|=0;

    $url = "https://$ip/$auth";
    $res = $ua->post($url,
		     [ login    => "$us",
		       password => "$pw",
		       action   => "Login",
		     ]);

    die("error $url:\n" . $res->code . " " . $res->message . "\n" . $res->content) unless ($res->is_redirect);
} else {
    die("unexpected $url:\n" . $res->code . " " . $res->message . "\n" . $res->content) unless $res->content =~ $floppy;
}

print "upload .. "; $|=1; $|=0;

$url = "https://$ip/$floppy";
$res = $ua->post($url,
  Content_Type => 'form-data',
  Content => [ vfloppy_file	=> [ $fn ],
	       action		=> "Upload",
	     ]);
die("error $url:\n" . $res->code . " " . $res->message) unless ($res->is_success);

if ($res->content !~ /image uploaded successfully/) {
    print "failed:\n" . $res->content;
    exit(1);
}

print "set mode .. "; $|=1; $|=0;

$res = $ua->post($url,
  Content_Type => 'form-data',
  Content => [ vfloppy_boot	=> $bo,
	       action		=> "Submit Changes",
	     ]);

die("error $url:\n" . $res->code . " " . $res->message) unless ($res->is_success);

if ($res->content !~ "\"$bo\" checked") {
    print "failed:\n" . $res->content;
    exit(1);
}

print "ok\n";
exit(0);
